@@ -27,25 +27,38 @@ class _MainUI():
     def __init__(self):
         self.mainui = MainUI()
         self.mainui.place()
-        self.source = 0
+        self.source = -1
         self.root_after = -1
         # face num for face_label
         self.lang_code = settings.lang_code
-        self.fxfy = None        
-        self.image_exts = ['jpg','png']
+        self.fxfy = None
+        self.image_exts = ['jpg','png', 'jpeg', 'webp']
         self.video_exts = ['mp4','avi','3gp','webm','mkv']
         self.showf_sv = None
         self.showfm = self.mainui.showframe
         self.infofm = self.mainui.infoframe
-        self.rbmixfm = self.mainui.rbmixframe        
+        self.rbmixfm = self.mainui.rbmixframe
         self.about_tl = None
         # vid
         self.vid = None
         self.vid_width = 0
         self.vid_height = 0
         self.vid_fps = 0
+
+        self.source_type = -1 # 0-> img , 1-> video
+
         self.cur_frame = None
-        self.pause = True
+        self.face_rects = []
+        self.picked_face_frames = []
+        self.showed_face_frames = []
+        self.show_size = (200,200)
+        self.zoom_in_size = (210,210)
+        self.save_size = (100,100)
+        self.zoomed_in_face_label = (0,0)
+        
+        self.doing = 'p'  # 'p'->'pick', r->'rec'
+
+        self.pause = False
         self.face_frames = []
         self.curf_index = 0
         # rec_result
@@ -135,11 +148,12 @@ class _MainUI():
             self.change_language )
         self.showfm.ct_entry.bind('<FocusOut>', None )
         self.showfm.pp_btn['command'] = self.pause_play
-        self.showfm.pick_btn['command'] = self.pick
+        self.showfm.rec_btn['command'] = self.recf_v0
+        self.showfm.pick_btn['command'] = self.pick_v0
         self.showfm.showf_go_btn['command'] = self.show_go
         self.showfm.showf_optionmenu_sv.trace('w', self.show_from )
-        self.infofm.prevf_btn['command'] = self.prevf
-        self.infofm.nextf_btn['command'] = self.nextf
+        # self.infofm.prevf_btn['command'] = self.prevf
+        # self.infofm.nextf_btn['command'] = self.nextf
         self.infofm.save_btn['command'] = self.savef
         self.rbmixfm.about_fn_btn['command'] = self.about_fn
         self.mainui.root.protocol("WM_DELETE_WINDOW", self.destroy )
@@ -148,27 +162,21 @@ class _MainUI():
         if self.vid is not None: self.vid.release()
         exit()
 
-# SHOW_FRAME FUNCTIONS 
-###############################################################################
-
     def pause_play( self, *args ):
+        if self.source_type != 1 : return
         if self.pause: 
-            if self.vid is None: return
-            if settings.data_empty(): 
-                if settings.debug: print('data is empty!')
-                self.show_data_empty_error()
-                return
-            self.root_after_cancel()
-            if self.cur_frame is None: return
-            self.recf()
-            self.showfm.pr_sv.set( _('Play') )
             self.pause = False
-            # self.pick()
-        else:
-            if self.cur_frame is None: return
             self.refresh_frame()
-            self.showfm.pr_sv.set( _('Recognize') )
+            self.showfm.pp_sv.set( _('Pause') )
+            if settings.debug:
+                print( 'Play. . .' )
+            
+        else:
+            self.root_after_cancel()
+            self.showfm.pp_sv.set( _('Play') )
             self.pause = True
+            if settings.debug:
+                print( 'Pause. . .' )
 
     def pick(self):
         if self.vid is None: return
@@ -191,7 +199,58 @@ class _MainUI():
         
         self.infofm.faces_text.delete(1.0,tk.END)
         self.change_face_show(0)
-    
+
+    def pick_v0(self):
+        
+        if self.doing == 'r':
+            self.clear_faces_frame()
+            self.doing ='p'
+
+        if (not self.pause) and self.vid :
+            _, self.cur_frame = self.vid.read()
+            
+        if self.cur_frame is None: return
+
+        self.cur_info_id = str(uuid.uuid4())
+        
+        gray_img=cv2.cvtColor(self.cur_frame,cv2.COLOR_BGR2GRAY)
+        self.face_rects = self.face_casecade.detectMultiScale(gray_img,1.3,5)
+        
+        if settings.debug:
+            print( self.face_rects )
+            print( type( self.face_rects ) )
+
+        for i in range( len( self.face_rects)):
+            self.add_face_label_p( i )
+
+
+    def add_face_label_p(self, num):
+        
+        x,y,w,h = self.face_rects[ num ]
+        _w = max( w, h )
+        
+        new_fl = Label( self.infofm.faces_frame )
+        frame = self.cur_frame[y:y+_w, x:x+_w]
+        frame = cv2.resize( frame, self.show_size )
+        vid_img = cv2.cvtColor( frame, cv2.COLOR_BGR2RGB )
+        vid_img = Image.fromarray( vid_img  )
+        imgtk = ImageTk.PhotoImage( image=vid_img )
+        new_fl.imgtk = imgtk
+        new_fl.configure(image=imgtk)
+        new_fl.bind("<Button-1>",lambda e: self.del_face_label_p(e , num) )
+
+        new_fl.pack(side=LEFT)
+
+        picked_face_frame = cv2.resize(self.cur_frame[y:y+h,x:x+w],self.save_size,\
+        interpolation=cv2.INTER_LINEAR)
+        self.picked_face_frames.append( picked_face_frame )
+
+    def del_face_label_p( self, e, num):
+        del self.picked_face_frames[ num ]
+        e.widget.destroy()
+        if settings.debug:
+            print( len(self.picked_face_frames) )
+
     def show_go( self, *args ):
         self.showf_sv = self.showfm.showf_sv.get()
         if len( self.showf_sv.strip() ) < 1: return
@@ -242,11 +301,16 @@ class _MainUI():
     
     def root_after_cancel( self ):
         if self.root_after != -1:
+            
+            _, self.cur_frame = self.vid.read()
+
             self.mainui.root.after_cancel( self.root_after )
             self.close_vid_cap()
             self.vid = None
         
     def play_video( self ):
+        self.pause = False
+        self.source_type = 1
         self.close_vid_cap()
         self.open_vid_cap()
         self.get_resize_fxfy()
@@ -255,12 +319,30 @@ class _MainUI():
     def refresh_frame(self):
         if self.vid == None: self.vid = cv2.VideoCapture( self.source )
         if not self.vid.isOpened(): self.show_nsrc_error(); return
-        ret, self.cur_frame = self.vid.read()        
-        self.cur_frame2label()
-        self.root_after = self.mainui.root.after( \
-            int(1000/self.vid_fps) , self.refresh_frame )
+
+        _, frame = self.vid.read()
+
+        gray_img=cv2.cvtColor( frame,cv2.COLOR_BGR2GRAY)
+        rects = self.face_casecade.detectMultiScale(gray_img,1.3,5)
+
+        for (x,y,w,h) in rects:
+            frame=cv2.rectangle( frame,(x,y),(x+w,y+h),(255,0,0),2)  
+        rects = None
+
+        vid_img = cv2.resize( frame , (0,0) , \
+            fx = self.fxfy, fy = self.fxfy )
+        vid_img = cv2.cvtColor( vid_img, cv2.COLOR_BGR2RGB )
+        vid_img = Image.fromarray( vid_img )
+        imgtk = ImageTk.PhotoImage( image=vid_img )
+        self.showfm.vid_frame_label.imgtk = imgtk
+        self.showfm.vid_frame_label.configure(image=imgtk)
+
+        if not self.pause:
+            self.root_after = self.mainui.root.after( \
+                int(1000/self.vid_fps) , self.refresh_frame )
 
     def view_image( self ):
+        self.source_type = 0
         self.cur_frame  = cv2.imread( self.face_src_path )
         frame = cv2.cvtColor( self.cur_frame, cv2.COLOR_BGR2RGB)
         img = Image.fromarray( frame )
@@ -294,13 +376,6 @@ class _MainUI():
         unable_open_s = _('Unable to open video source')
         messagebox.showerror( unable_open_s, unable_open_s+': '+self.showf_sv )
 
-
-###############################################################################
-# SHOW_FRAME FUNCTIONS END
-
-# INFO_FRAME FUNCTIONS
-###############################################################################
-
     def prevf( self ):
         self.change_face_show(-1)
         pass
@@ -317,7 +392,7 @@ class _MainUI():
         img_path =os.path.join( settings.faces_path , self.cur_info_id )
         os.makedirs(img_path,exist_ok=True )
         count = 0
-        for f in self.face_frames:
+        for f in self.picked_face_frames:
             cv2.imwrite( f'{img_path}/{count}.png' , f)
             count+=1
         self.cur_info_id = None
@@ -328,12 +403,13 @@ class _MainUI():
         self.infofm.num_label['text'] = \
         f'{self.curf_index+1}/{len(self.face_frames)+len(self.recfs)-1}'
             
-    def change_face_show(self, _as):
+    def change_face_show(self, _as, rec = True):
         if len(self.face_frames) >0:
             # NEW
             self.curf_index += _as
             self.curf_index = 0 if self.curf_index < 0 else self.face_enter_count-1\
             if self.curf_index >= self.face_enter_count else self.curf_index 
+
             vid_img = cv2.cvtColor( self.face_frames[ self.curf_index ], \
             cv2.COLOR_BGR2RGB )
             vid_img = Image.fromarray( vid_img )
@@ -378,20 +454,136 @@ class _MainUI():
         if settings.debug: 
             print('self.cur_frame not None')
         self.rec_gray_img=cv2.cvtColor(self.cur_frame,cv2.COLOR_BGR2GRAY)
-        self.recfs=self.face_casecade.detectMultiScale(self.rec_gray_img,1.3,5)
-        if len( self.recfs ) < 1: return
+        self.face_rects=self.face_casecade.detectMultiScale(\
+        self.rec_gray_img,1.3,5)
+        if len( self.recfs ) < 1: return        
+        
         self.change_face_show(0)
+
         for (x,y,w,h) in self.recfs:
             self.cur_frame=cv2.rectangle(\
             self.cur_frame,(x,y),(x+w,y+h),(255,0,0),2)  
+
         self.cur_frame2label()
+    
+    def restore_face_label_size( self, index ):
+        label, index = self.zoomed_in_face_label
+        if not label.winfo_exists(): return
+        
+        frame =  self.showed_face_frames[ index ]
+        frame = cv2.resize(frame, self.show_size )
+
+        vid_img = cv2.cvtColor( frame, cv2.COLOR_BGR2RGB )
+        vid_img = Image.fromarray( vid_img  )
+        imgtk = ImageTk.PhotoImage( image=vid_img )
+        label.imgtk = imgtk
+        label.configure(image=imgtk)
+
+
+    def show_info( self, label, _id, index):
+
+        if (self.zoomed_in_face_label[0]!=0) and \
+        (self.zoomed_in_face_label[0] != label) :
+            self.restore_face_label_size( index )
+
+        frame =  self.showed_face_frames[ index ]
+        frame = cv2.resize(frame, self.zoom_in_size )
+
+        vid_img = cv2.cvtColor( frame, cv2.COLOR_BGR2RGB )
+        vid_img = Image.fromarray( vid_img  )
+        imgtk = ImageTk.PhotoImage( image=vid_img )
+        label.imgtk = imgtk
+        label.configure(image=imgtk)
+
+        self.zoomed_in_face_label = (label, index)
+
+        info_file_path = os.path.join( \
+        settings.infos_path,  _id )
+        self.infofm.faces_text.delete(1.0,tk.END)
+        
+        if not os.path.exists( info_file_path ): 
+            self.infofm.faces_text.insert('1.0', _('No informations found') )
+
+        self.infofm.faces_text.insert('1.0', \
+        open( info_file_path, 'r' ).read() )
+
+
+    def add_face_label_r( self, num ):
+
+        index = len( self.showed_face_frames )
+
+        new_fl = Label( self.infofm.faces_frame )
+
+        x,y,w,h = self.face_rects[ num ]
+        roi_gray= self.rec_gray_img[y:y+h,x:x+w]
+        roi_gray= cv2.resize( roi_gray, self.save_size,\
+        interpolation=cv2.INTER_LINEAR)
+    
+        result=self.recognizer.predict(roi_gray)
+
+        _id = self.info_ids[result[0]]
+        _h = max( h, w )
+        frame = self.cur_frame[y:y+_h,x:x+_h]
+        frame = cv2.resize(frame, self.show_size )
+
+        self.showed_face_frames.append( frame )
+
+        vid_img = cv2.cvtColor( frame, cv2.COLOR_BGR2RGB )
+        vid_img = Image.fromarray( vid_img  )
+        imgtk = ImageTk.PhotoImage( image=vid_img )
+        new_fl.imgtk = imgtk
+        new_fl.configure(image=imgtk)
+
+        new_fl.bind("<Double-Button-1>",lambda e: \
+        self.del_face_label_r(e, index))
+
+        new_fl.bind("<Button-1>",lambda e: self.show_info(new_fl , _id, index) )
+
+        new_fl.pack(side=LEFT)
+
+        self.show_info(new_fl , _id, index)
+
              
-###############################################################################
-# INFO_FRAME FUNCTIONS  END
+    def del_face_label_r( self, e, index):
+        if self.zoomed_in_face_label[0] == e.widget:
+            self.zoomed_in_face_label= (0,0)
+        e.widget.destroy()
+        self.showed_face_frames[index] = None
+    
+    def clear_faces_frame( self ):
+        for child in self.infofm.faces_frame.winfo_children():
+            child.destroy()
 
-# LANGCOMBOBOX FUNCTIONS
-###############################################################################
 
+    def recf_v0(self):
+
+        if self.source_type == -1:return
+        if (not self.pause) and self.vid:
+            _, self.cur_frame = self.vid.read()
+        
+        if settings.data_empty():
+            self.show_data_empty()
+            return
+        
+        if self.doing == 'p':
+            self.clear_faces_frame()
+            self.doing ='r'
+
+        if self.cur_frame is None: return
+
+        self.rec_gray_img=cv2.cvtColor(self.cur_frame,cv2.COLOR_BGR2GRAY)
+        self.face_rects=self.face_casecade.detectMultiScale(\
+        self.rec_gray_img,1.3,5)
+
+        if len( self.face_rects ) < 1: 
+            if settings.debug:
+                print('len( self.face_rects ) < 1 ')
+            return
+
+        for i in range( len(self.face_rects)) :
+            self.add_face_label_r(i)
+            
+             
     def change_language(self, lang ):
 
         lang_display_name = self.rbmixfm.lang_combobox_var.get()
@@ -412,19 +604,11 @@ class _MainUI():
             os.execl(sys_executable, sys_executable, * sys.argv)
         pass
 
-###############################################################################
-# LANGCOMBOBOX FUNCTIONS END
-
-# OTHER FUNCTIONS
-###############################################################################
     def show_nsrc_error( self ):
         unable_open_s = _('Unable to open video source')
         messagebox.showerror(  unable_open_s, unable_open_s+': '+self.showf_sv )
-    def show_data_empty_error( self ):
+    def show_data_empty( self ):
         unable_open_s = _('Nothing enter')
         msg  = _("You haven't entered anything yet! ")
         messagebox.showerror(  unable_open_s, msg, )
-
-###############################################################################
-# OTHER FUNCTIONS END
-
+        
\ No newline at end of file
